//-------------------------------------------------------------------
// Heartbeat.cpp
//
// 3/15/2002
// Applied Data Systems
//
// This application blinks LED1 on ADS development systems
// By default it will run infinitely, but a command line parameter
// can be used to set a duration roughly in seconds.
// 
// Revision history:
//   3/15/02 : CBT - Created
//   3/28/02 : AK  - heartbeat message only displays in debug release
//                 - add initial message about app
//                 - (project file) debug release is heartbeat_d.exe
// 11apr03 720020-11013 ak
//  - restructure code to support Bitsy/Bitsy Plus (COM1: DTR line)
//
// This software is provided as-is with no warranty
//-------------------------------------------------------------------

#include "stdafx.h"

#define LED_ON			1
#define LED_OFF			0

TCHAR	szLEDName[]		= _T("LED1:");
TCHAR	szCOMName[]		= _T("COM1:");
TCHAR	szPortName[5];


HANDLE	hPort			= INVALID_HANDLE_VALUE;
BYTE	ledStatus;
DWORD	dwSize;
DCB		COM1dcb;

void	(*LED_On)(), (*LED_Off)();


//-------------------------------------------------------------------
// Functions to turn LED on/off
//-------------------------------------------------------------------
void COM1_DTR_On ()
{
	COM1dcb.fDtrControl = DTR_CONTROL_DISABLE;
	SetCommState(hPort, &COM1dcb);
}

void COM1_DTR_Off ()
{
	COM1dcb.fDtrControl = DTR_CONTROL_ENABLE;
	SetCommState(hPort, &COM1dcb);
}

void LED1_On ()
{
	ledStatus = LED_ON;
	WriteFile(hPort, &ledStatus, 1, &dwSize, NULL);
}

void LED1_Off ()
{
	ledStatus = LED_OFF;
	WriteFile(hPort, &ledStatus, 1, &dwSize, NULL);
}

//-------------------------------------------------------------------
// Main
//-------------------------------------------------------------------
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	int		iIterations	= _ttoi(lpCmdLine);  // read command line parameter
	BOOL	BeatIndefinitely, UseLEDDriver;
	

	// 1. Initialize
	// 1.1 Display startup message
//	BeatIndefinitely = (iIterations == 0) || (iIterations < -1);
	BeatIndefinitely = (iIterations <= 0);
	if(BeatIndefinitely)
	{
		RETAILMSG(TRUE, (_T("* Heartbeat.exe: Blinking LED indefinitely, about once per second...\r\n")));
		iIterations = 1; // any value > 0 will work
	}
	else
		RETAILMSG(TRUE, (_T("* Heartbeat.exe: Blinking LED %n times, about once per second...\r\n"),iIterations));


	// 1.2 Determine LED driver to use
	// First, see if possible to open LED driver
	hPort = CreateFile(szLEDName, GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);
	UseLEDDriver= (hPort != INVALID_HANDLE_VALUE);

	// 1.3 Initialize drivers for appropriate product
	if (UseLEDDriver)
	{
		RETAILMSG(TRUE, (_T("  Using LED1: driver.\r\n")));
		LED_On  = &LED1_On;
		LED_Off = &LED1_Off;
	}
	else // if not, use serial port
	{
		hPort = CreateFile(szCOMName, GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);
		if (hPort == INVALID_HANDLE_VALUE)
		{
			RETAILMSG(TRUE, (_T("! Error %d: Unable to open %s or %s. Program aborted.\r\n"),GetLastError(),szLEDName,szCOMName));
			return -1;
		}

		// Get COM1: parameters
		COM1dcb.DCBlength = sizeof(DCB);
		if(!GetCommState(hPort, &COM1dcb))
		{
			RETAILMSG(TRUE, (_T("! Error %d: Unable to get state of %s. Program aborted.\r\n"),GetLastError(),szCOMName));
			CloseHandle(hPort);
			return -1;
		}

		RETAILMSG(TRUE, (_T("  Using DTR line of COM1:.\r\n")));
		LED_On  = &COM1_DTR_On;
		LED_Off = &COM1_DTR_Off;
	}


	// 2. "Beat" the LED

	// start with LED off
	LED1_Off();

	while (iIterations > 0)
	{
	    DEBUGMSG(TRUE, (_T("heartbeat\r\n")));

		LED_On();		// thump...
		Sleep(150);		
		LED_Off();
		Sleep(100);

		LED_On();		// thump...
		Sleep(200);		
		LED_Off();

		Sleep(500);		// pause....

		if (!BeatIndefinitely)
			iIterations--;
	}


	// 3. Close down application

	// Leave LED On (closing driver may turn it off)
	LED_On();

	CloseHandle(hPort);

	RETAILMSG(TRUE, (_T("\r\nHeartbeat complete.\r\n")));

	return 0;

}